from types import FunctionType, MethodType

class Meta(type):
    class FunctionOverride:
        def __init__(self,cls_name,fn_name,fn):
            self.__cls_name__=cls_name
            self.__name__=fn_name
            self.__fn__= fn
        
        def __call__(self,*args, **kwargs):
            return self.__fn__(self.__cls_name__,self.__name__,*args,**kwargs)

    def __new__(cls, name, bases, dct):
        attrs = {
            "__name__":name,
            "__context__":{} if '__context__' not in dct else dct['__context__'],
            "__dynamic_methods__":[],
            "__set_hook__":cls.__set_hook_method__,
            "__hook_method__":dct["__hook_method__"] if '__hook_method__' in dct else None,
            "FunctionOverride":cls.FunctionOverride
        }      
        for key, value in dct.items():
            if key != "__construct__" and key != "__dynamic_methods__" and key!="__hook_method__":
                    attrs[key]=value
            '''
            if key not in attrs:
                if key != "__construct__":
                    attrs[key]=value
            '''
            if key=="__construct__":
                if '__hook_method__' in dct:
                    attrs["__hook_method__"]=dct["__hook_method__"]
                    if value:
                        for o in bases:
                            if(isinstance(o,Meta)):
                                o.Meta.__construct__= value
                                for item in o.Meta.__construct__:
                                    for k,v in item.items():
                                        attrs[k]=type(k,(dynamic_creator,),{"__dynamic_methods__":v,        "__hook_method__":attrs['__hook_method__']})()
                else:
                    raise(Exception("missing field '__hook_method__'."))
            if key=="__dynamic_methods__":
                for item in value:
                    attrs["__dynamic_methods__"].append(item)
                    attrs[item]= cls.FunctionOverride(name,item,attrs["__hook_method__"])
        return super(Meta,cls).__new__(cls, name, bases, attrs)

    def __set_hook_method__(self,fn):
        if fn:
            self.__hook_method__= fn
            for k in self.__dynamic_methods__:
                self.__dict__[k]=self.FunctionOverride(self.__name__,k,fn)

class dynamic_creator(metaclass=Meta):
   
    def __init__(self,fn=None):
        self.__set_hook__(fn)
        super(dynamic_creator,self).__init__()
        
    class Meta:
        __name__=None
        __construct__=None
        __hook_method__=None


#! python 3.6

#########################################################
#| @File    :   standard_result.py
#| @ClsName :   standard_result
#| @Version :   1.0.0
#| @History :
#|----------------------------------------------------------------------------------------------------------|
#| version       | Type                | Author        | Contact                     | Time                 |
#|----------------------------------------------------------------------------------------------------------|
#| 1.0.0         | Create              | Tony.Don      | yeihizhi@163.com            | 2022-01-27 20:34:44  |
#|----------------------------------------------------------------------------------------------------------|
#| @License :   BSD-3-Clause, Copyright (c) <2021>, <Tang Jing>
#| @Desc    :   None.
#########################################################

# import lib by pypi
# import lib by project

#code start


class standard_result(object):
    state = False
    code = 0
    msg = ""
    data = None
    _hook_log_ = []
    _log_handle_ = None

    @classmethod
    def set_log_handle(self, handle):
        '''
        set logging handle
        
        args:
            handle: logging handle.
        '''
        self._log_handle_ = handle

    @classmethod
    def set_log_hook(self, hooks={}):
        '''
        hook state auto write log.
        
        args:
            hooks: {state_code:code}
        '''
        self._hook_log_ = hooks

    @classmethod
    def append_log_hook(self, state, code=None, log_type="info"):
        '''
        append an log hook config.
        
        args:
            state: Boolen.
            code: user define state code.
            log_type: str, emun(info,error,warning,debug)
        '''
        self._hook_log_[state+"_"+code if code else ""]= {"code":code,"log_type":log_type.lower()}

    @classmethod
    def log_fiter(self):
        '''
        auto write log.
        '''
        if self.state+"_"+self.code if self.code else "" in self._hook_log_:
            if self._log_handle_:
                method= getattr(self._log_handle_, self._hook_log_[self.state+"_"+self.code if self.code else ""]['log_type'].lower())
                if method:
                    log_formatter=""
                    method(log_formatter)
                else:
                    raise Exception("Log handle method '%s' can not found."% self._hook_log_[self.state+"_"+self.code if self.code else ""]['log_type'].lower())
            else:
                raise Exception('Log handle is none.')
    
    @classmethod
    def dumps(self, state, code, msg, data):
        '''
        dumps result.

        args:
            state: <Boolen>
            code: <int>, user define state code
            msg: <str>, message
            data: <object>, return data.
        '''
        self.state = (state,)
        self.code = (code,)
        self.msg = (msg,)
        self.data = (data,)
        return self.state, {
            "code": self.code,
            "msg": self.msg,
            "data": self.data,
        }

    @classmethod
    def loads(self, state, result):
        '''
        make result transformation to object.

        args:
            state:<Boolen>
            result:<dict>
        '''
        assert "code" in result, "result must have 'code' key."
        assert "msg" in result, "result must have 'msg' key."
        assert "data" in result, "result must have 'data' key."
        self.state = state
        self.code = result["code"]
        self.code = result["msg"]
        self.data = result["data"]
        return self


class web_result(standard_result):
    state = False
    code = 200
    msg = "success"
    data = {}



import threading
import re
from TDhelper.network.socket import base,SOCKET_TYPE,SOCKET_EVENT, Event,trigger,call, get_host_ip
import socket

class Client(base,threading.Thread):
    def __init__(self,ip,port,buffSize=1024,maxReconnect=5):
        threading.Thread.__init__(self)
        super(Client,self).__init__()
        if re.match(r"^(?=^.{3,255}$)[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})+$", ip, re.I | re.M):
            ip= socket.gethostbyname(ip)
        self._max_reconnect= maxReconnect
        self._reconnect=0
        self.uri=(ip,port)
        self.__runing=threading.Event()
        self._buffSize=buffSize
        self.__state=True
        self.createsocket(SOCKET_TYPE.TCPIP)
        self.setTimeout(10)
        self.__runing.set()

    def run(self):
        self.__connection()

    @trigger("connection")
    def __connection(self):
        try:
            self.__state= True
            self.connection(self.uri)
            self._reconnect= 0
            self.on(SOCKET_EVENT.onConnection,self)
        except Exception as e:
            self.__state=False
            if self._reconnect>self._max_reconnect:
                self.on(SOCKET_EVENT.onError,e)
            else:
                self._reconnect+=1
                self.__connection()

    @trigger("send")
    def sendMsg(self,buff):
        try:
            if self.__state:
                self.send(self.getSocket(),buff)
        except Exception as e:
            self.on(SOCKET_EVENT.onError,e)

    @trigger("recv")
    def recvMsg(self):
        try:
            while self.__runing.is_set() and self.__state:      
                buff=self.recv(self.getSocket(),self._buffSize)
                if not buff:
                    self.__runing.clear()
                    break
                self.on(SOCKET_EVENT.onRecv,buff)
        except Exception as e:
            pass
            #self.on(SOCKET_EVENT.onError,e)
        finally:
            pass #self.closeClient()

    def setRuning(self):
        if self.__runing.is_set():
            self.__runing.clear()

    def closeClient(self):
        try:
            if self.__state:
                self.__state=False
                self.setRuning()
                self.close()
                #super(Client,self).close()
        except Exception as e:
            self.on(SOCKET_EVENT.onError,e)


import hashlib
from generic.transformationType import transformation

def MD5(s:"str|list|dict",encoding="utf-8"):
    m= hashlib.md5()
    if isinstance(s,str):
        m.update(s.encode(encoding))
    elif isinstance(s,dict):
        _str=""
        for k,v in s.items():
           _str="".join([_str,transformation(v,str)])
        m.update(_str.encode(encoding))
    elif isinstance(s,list):
        _str=""
        for v in s:
            _str="".join([_str,transformation((lambda v:v if not isinstance(v,tuple) else v[1])(v),str)])
        m.update(_str.encode(encoding))
    else:
        raise Exception("MD5 input param type had error, str,dict,list.")
    return m.hexdigest()

def SALT(s:"list|dict",encoding="utf-8",desc=True):
    '''SALT
        
        generate salt.
    
    Args:
    
        s (list|dict): input value.
        
        encoding(encode): encode, default 'UTF-8'.
        
        desc(Boolen): sort type, default true. value set true sort with descending order, value set false sort with ascending order    
        
    Returns:
            
            salt, 128 bit 
        
    Raises:
            INPUT Type ERROR
        
    '''
    if isinstance(s,list):
        return MD5(s,encoding)
    elif isinstance(s,dict):
        return MD5(sorted(s.items(), reverse=desc), encoding)
    else:
        raise Exception("SALT input param type had errot, list,dict.")


import logging
import html
from TDhelper.cache.memcache import cache

class RPC_Server:
    __logger__ = logging

    def __init__(self, logger_handle=None):
        self.__logger__ = logger_handle if logger_handle else self.__logger__

    def __insert__(self, k, v):
        try:
            cache.insert(k, v)
        except Exception as e:
            self.__logger__.error(e)

    def __append__(self, k, v):
        try:
            cache.append(k, v)
        except Exception as e:
            self.__logger__.error(e)

    def __set__(self, k, v):
        try:
            cache.update(k, v)
        except Exception as e:
            self.__logger__.error(e)

    def __get__(self, k=None):
        try:
            if k:
                return cache.findValue(k)
            else:
                return cache.__get__()
        except Exception as e:
            self.__logger__.error(e)

    def __delete__(self, k):
        try:
            cache.destory(k)
        except Exception as e:
            self.__logger__.error(k)

    def __clear__(self):
        cache.__clear__()
    
    def __transfer_method_type__(self,v:int):
        if v==1:
            return "GET"
        elif v==2:
            return "POST"
        elif v==3:
            return "PUT"
        elif v==4:
            return "DELETE"
        else:
            raise "method transfer value must 1,2,3,4."
    
    def add_service(self, k, n, d, s, p):
        """add service
            add new service
        Parameters:
            k - <class:string>, service key.
            n - <class:string>, service name.
            d - <class:string>, service description.
            s - <class:stirng>, service secret.
            p - <class:string>, service protocol
        Returns:
             None
        Raises:
             None
        """
        cnf = {
            "key": k,
            "name": n,
            "description": d,
            "secret": s,
            "protocol": p,
            "hosts": [],
            "methods": {}
        }
        self.__insert__(k, cnf)

    def add_host(self, service, serverId, host, port, state, proto):
        cnf = {
            "serverId": serverId,
            "host": host,
            "port": port,
            "state": state,
            "proto": proto
        }
        cache.findInstance("".join([service,'.hosts'])).append(cnf)

    def add_method(self, service,key,uri,method,version,description="",collect=''):
        key= key.split('.')
        if len(key)>1:
            key=key[1]
        else:
            key=key[0]
        cnf = {
            "collect": collect.upper() if collect else collect,
            "key": key.upper() if key else key,
            "uri": uri,
            "method": self.__transfer_method_type__(method),
            "version": version,
            "description": description,
            "params": [],
            "returns": {}
        }
        cache.insert("".join([service, ".methods.",key]),cnf)
        
    def add_param(self,service,method,key,desc,defaultValue=None):
        cnf={
                    "key": key,
                    "description": desc,
                    "defaultValue":defaultValue
            }
        cache.findInstance("".join([service,".methods.",method,".params"])).append(cnf)

    def add_return(self,service,method,v_type,examples):
        cnf={
                "valueType": v_type,
                "examples": html.unescape(examples),
                "descriptions": []
            }
        cache.update("".join([service,".methods.",method,".returns"]),cnf)
        
    def add_return_desc(self,service,method,key,desc):
        cnf={
                "key": key,
                "valueDescription": desc
            }
        cache.findInstance("".join([service,".methods.",method,".returns",".descriptions"])).append(cnf)


import json
import random
import hashlib


def xWwwFormUrlencoded(v: dict, encoding='utf-8', headers: dict = None):
    '''
        if http post content-type is application/x-www-form-urlencoded, then you can use this function to translate post data chunk.

        params:
        - v: <dict>. your post data, value formatter(n1=x1&n2=x2...).
        - encoding: <str>. encoding type, default value is utf-8.  

        returns:
        - headers:<dict>. http header.
        - data: <str>. post data string.
    '''
    try:
        m_v = ''
        count = 0
        for k, k_v in v.items():
            m_v += k+"="+k_v
            count += 1
            if count < len(v):
                m_v += "&"
        if headers:
            headers["Content-type"] = "application/x-www-form-urlencoded; charset= %s" % encoding
        else:
            headers = {
                "Content-Type": "application/x-www-form-urlencoded; charset= %s" % encoding}
        data = m_v.replace("+", "%2B")
        data = data.encode(encoding)
        return headers, data
    except Exception as e:
        raise e


def formData(v: dict, encoding='utf-8', headers: dict = None):
    '''
        if http post content-type is multipart/form-data, then you can use this function to translate post data chunk.

        params:
        - v: <dict>. your post data.
        - encoding: <str>. encoding type, default value is utf-8.

        returns:
        - header: <dict>. 
        - data: <json> 
    '''
    try:
        token = str(random.randint(0, 100000))
        m_md5 = hashlib.md5()
        m_md5.update(token.encode(encoding))
        token = m_md5.hexdigest()
        boundaryStr = "----WebKitFormBoundary%s" % token
        if headers:
            headers["Content-type"] = "multipart/form-data; boundary=%s" % boundaryStr
            headers["cache-control"] = "no-cache"
        else:
            headers = {
                'Content-Type': "multipart/form-data; boundary=%s" % boundaryStr,
                'cache-control': "no-cache",
            }
        startStr = '--'+boundaryStr+'\r\n'
        dataDispositionFormatter = "Content-Disposition: form-data; name=\"{0}\"\r\n\r\n{1}\r\n"
        endStr = '--'+boundaryStr+'--'
        data = ''
        if isinstance(v, dict):
            for item, value in v.items():
                data += dataDispositionFormatter.format(item, value)
            data = startStr + data + endStr
            return headers, data
        else:
            raise Exception("parameter 'v' type must is dict.")
    except Exception as e:
        raise e


def jsonData(v: dict, encoding: str = 'utf-8', headers: dict = None):
    '''
        if http post content-type is application/json, then you can use this function to translate post data chunk.

        params:
        - v: <dict>. your post data.
        - encoding: <str>. encoding type, default value is utf-8.

        returns:
        - header: <dict>. 
        - data: <json> 
    '''
    try:
        if headers:
            headers["Content-Type"] = "application/json;charset=%s" % encoding
        else:
            headers = {"Content-Type": "application/json;charset=%s" % encoding}
        return headers, json.dumps(v)
    except json.JSONDecodeError as e:
        raise e


def xmlData(v: dict, encoding: str = 'utf-8'):
    '''
        wait for, void function.
    '''
    pass


if __name__ == '__main__':
    dictData = {'name': 'tangjing', 'pwd': '123456'}
    strData = "name=tangjing&pwd=123456"

    header, data = xWwwFormUrlencoded(dictData)
    print(header)
    print(data)



import abc
import six

@six.add_metaclass(abc.ABCMeta)
class cacheInterface:
    __cache__={}
    __cursor__=None
    @abc.abstractclassmethod
    def __init__(self):
        pass
    
    @abc.abstractclassmethod
    def set(self,*args,**kwargs):
        pass
        
    @abc.abstractclassmethod
    def get(self,flag='single',*args,**kwargs):
        pass
    
    @abc.abstractclassmethod
    def exist(self,*args,**kwargs):
        pass
    
    @abc.abstractclassmethod
    def collect(self,*args,**kwargs):
        pass
    
    @abc.abstractclassmethod
    def addCollect(self,k,v):
        pass
    
    @abc.abstractclassmethod        
    def delCollect(self,k):
        pass
    
    @abc.abstractclassmethod   
    def remove(self,*args,**kwargs):
        pass
    
    @abc.abstractclassmethod   
    def update(self,*args,**kwargs):
        pass


from TDhelper.network.rpc.Generic.Host import HostManager
from TDhelper.Decorators.log import logging, logging_setup

class attribute_override:
        def __init__(self, name):
            self.__name__ = name

        def __set__(self, instance, v):
            if isinstance(instance.__rpc_server_cnf__,dict):
                if isinstance(v, str):
                    instance.__rpc_server_cnf__[self.__name__] = v.strip('/')
                else:
                    raise Exception("%s uri must str type." % self.__name__)
            else:
                raise Exception("rpc server config must dict. checked init method parameter server_cnf.")

        def __get__(self, instance, owen):
            if isinstance(instance.__rpc_server_cnf__,dict):
                if self.__name__ in instance.__rpc_server_cnf__:
                    return instance.__rpc_server_cnf__[self.__name__].strip('/')+"/"
                else:
                    raise Exception("rpc server config not found %s."%self.__name__)
            else:
                raise Exception("rpc server config must dict. checked init method parameter server_cnf.")

        def __delete__(self, instance):
            instance.__rpc_server_cnf__.pop(self._name)

class Meta(type):
    def __new__(cls, name, bases, dct):
        attrs={
            "__host__": HostManager(),
            "__RPC_SERVICES__":None,
            "__RPC_CNF__":None,
            "__ENCODING__":"utf-8",
            "__headers__":{},
            "__logger_hander__":logging,
            "__rpc_server_cnf__":{},
            "__service_register_uri__":attribute_override("service"),
            "__host_register_uri__":attribute_override("host"),
            "__method_register_uri__":attribute_override("method"),
            "__params_register_uri__":attribute_override("params"),
            "__return_register_uri__":attribute_override("return"),
            "__return_desc_register_uri__":attribute_override("return_desc")
        }
        for k,v in dct.items():
            if k not in attrs:
                attrs[k]=v
        return super(Meta,cls).__new__(cls,name,bases,attrs)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import bson
import copy
from TDhelper.db.mongodb.dbhelper import dbhelper
'''
class\r\n
    objectId
description\r\n
    mongodb's bson.objectid\r\n
'''
class objectId(dbhelper):
    model=None
    def __init__(self):
        super(objectId,self).__init__()
        self.setCollection(type(self).__name__)
        self.model={
            "_id":None
        }
    @property
    def oId(self):
        if self.model["_id"]:
            return self.model["_id"]
        else:
            self.model["_id"]=bson.objectid.ObjectId()
            return self.model["_id"]
    @oId.setter
    def oId(self,args):
        if args:
            self.model["_id"]=bson.objectid.ObjectId(args)
        else:
            self.model["_id"]=bson.objectid.ObjectId()
        
    def toSave(self):
        if self.model:
            if self.oId:  
                return self.save(self.model)
        else:
            return None

    def deleteById(self):
        if self.model:
            return self.remove({'_id':self.oId})

    def getbyId(self):
        if self.model:
            result=copy.deepcopy(self)
            if result:
                oResult=self.findOne({'_id':self.oId})
                if oResult:
                    result.model=oResult
                    return result
                return None
        return None

    def getByfield(self, field_name):
        if field_name:
            return self.findOne({field_name: self.model[field_name]})
        return None

    def UpdateById(self):
        if self.oId:
            self.update({'_id':self.oId}, self.model)

    def getOneByQuery(self, query):
        if self.model:
            result=copy.deepcopy(self)
            if result:
                oResult=self.findOne(query)
                if oResult:
                    result.model=oResult
                    return result
        return None


import datetime
from enum import Enum
from TDhelper.Event.Event import Event
from TDhelper.Spider.regex import Analysis
from TDhelper.Spider.models.spider_event import event as Analysis_Event
from TDhelper.Spider.models.status import STATUS as SPIDER_STATUS
from TDhelper.Spider.models.fingerprint import fingerprint
from TDhelper.Cache.pools import pools
from TDhelper.Spider.models.Cache_L1 import L1, event as L1_EVENT, L2
from TDhelper.Spider.models.BadRequestModel import BadRequest
from threading import Thread

import copy


class event(Enum):
    onListen = 'onlisten'


class spiderPools(pools):
    def __init__(self, rulesConfig, pool_length=10, cache_size=50, fingerprint=True):
        super(spiderPools, self).__init__(pool_length)
        self._fingerprint = fingerprint  # 重复检查开关
        self._rulesConfig= rulesConfig
        self._cache = L1(cache_size)
        self._exclude_url = {}
        self._cache.registerEvent(L1_EVENT.onPush, self.onPush)
        self._debug = False
        self._thread_state= False
        # 初始化爬虫线程池
        for i in range(0, pool_length):
            m_spider = Analysis(rulesConfig)
            m_spider.registerEvent(
                Analysis_Event.onIndexComplete, self.onIndexComplete)
            m_spider.registerEvent(
                Analysis_Event.onListComplete, self.onListComplete)
            m_spider.registerEvent(Analysis_Event.onDebug, self.onDebug)
            m_spider.registerEvent(
                Analysis_Event.onDetailComplete, self.onDetailComplete)
            m_spider.registerEvent(Analysis_Event.onError, self.onError)
            m_spider.registerEvent(
                Analysis_Event.onFingerprintComplete, self.onFingerprintComplete)
            self.push(m_spider)

    def serach(self,key=None):
        if key:
            m_search = Analysis(self._rulesConfig)
            m_search.registerEvent(
                Analysis_Event.onIndexComplete, self.onIndexComplete)
            m_search.registerEvent(
                Analysis_Event.onListComplete, self.onListComplete)
            m_search.registerEvent(Analysis_Event.onDebug, self.onDebug)
            m_search.registerEvent(
                Analysis_Event.onDetailComplete, self.onDetailComplete)
            m_search.registerEvent(Analysis_Event.onError, self.onError)
            m_search.registerEvent(
                Analysis_Event.onFingerprintComplete, self.onFingerprintComplete)
            m_search.search([key])

    def pushCache(self, value):
        self._cache.push(value[0], value[2], value[1])

    @property
    def State(self):
        return self._thread_state
    @property
    def ExcludeUrl(self):
        return self._exclude_url

    @ExcludeUrl.setter
    def ExcludeUrl(self,value):
        self._exclude_url= value

    def getCache(self):
        return self._cache.getCache()

    def onPush(self, *args, **kwargs):
        self._lock.acquire()
        if self.available_resources > 0:
            self._thread_state= True
            self.__startSpiderThread()
        self._lock.release()

    def __startSpiderThread(self):
        thread_spider = Thread(target=self.__process)
        thread_spider.start()

    def listen(self, *args, **kwargs):
        '''向外部抛出监听事件'''
        return self.on(event.onListen, *args, **kwargs)

    def __process(self):
        # 爬虫线程
        process_spider = self.pop()
        if process_spider:
            m_state = True
            while m_state:
                m_url = self._cache.pop()
                if m_url:
                    if m_url[0]:
                        # 如果内容未更改则跳过；并认为后续翻页内容均为未更改内容,并同时跳过后续翻页爬取.
                        if m_url[1]:
                            if m_url[1] in self._exclude_url:
                                continue
                        try:
                            self._debug= False
                            process_spider.start(m_url[2], m_url[0])
                        except Exception as e:
                            pass
                        if process_spider.getStatus == SPIDER_STATUS.SPIDER_FINGERPRINT_IS_REPEAT:
                            if m_url[1]:
                                self._exclude_url[m_url[1]
                                                ] = process_spider.getCurrentUrl
                    else:
                        m_state = False
                else:
                    self._thread_state= False
                    m_state = False
            self.push(process_spider)
        else:
            pass

    # 事件监听
    def onIndexComplete(self, *args, **kwargs):
        if len(args) > 0:
            m_analysis = args[0]
            if m_analysis.getStatus == SPIDER_STATUS.SPIDER_SUCCESS:
                data = m_analysis.getData
                if data:
                    for item in data['data']:
                        self.pushCache(
                            [item['url'], m_analysis.getNextUrl, data['analysis_key']])
                    try:
                        if not self._debug:
                            self._lock.acquire()
                            self.on(event.onListen, *args, **kwargs)
                    except Exception as e:
                        pass
                    finally:
                        if not self._debug:
                            self._lock.release()

    def onListComplete(self, *args, **kwargs):
        if len(args) > 0:
            m_analysis = args[0]
            if m_analysis.getStatus == SPIDER_STATUS.SPIDER_SUCCESS:
                m_data = m_analysis.getData
                if m_data['data']:
                    # TODO
                    for item in m_data['data']:
                        # 为了解决自动判断页面抓取重复取消下一页，此处存的来源为下一页
                        self.pushCache(
                            [item['url'], m_analysis.getNextUrl, m_data['analysis_key']])
                    try:
                        if not self._debug:
                            self._lock.acquire()
                            self.on(event.onListen, *args, **kwargs)
                    except Exception as e:
                        pass
                    finally:
                        if not self._debug:
                            self._lock.release()

            if m_analysis.getNextUrl:
                self.pushCache(
                    [m_analysis.getNextUrl, m_analysis.getCurrentUrl, m_data['analysis_key']])

    def onDetailComplete(self, *args, **kwargs):
        if len(args) > 0:
            m_analysis = args[0]
            if m_analysis.getStatus == SPIDER_STATUS.SPIDER_SUCCESS:
                m_data = m_analysis.getData
                try:
                    if not self._debug:
                        self._lock.acquire()                    
                        self.on(event.onListen, *args, **kwargs)
                except Exception as e:
                    pass
                finally:
                    if not self._debug:
                        self._lock.release()

    def onDebug(self, *args, **kwargs):
        try:
            self._lock.acquire()
            self._debug = args[0].debug
            if kwargs['event'] == Analysis_Event.onFingerprintComplete:
                pass #调试模式不检查重复 self.onFingerprintComplete(*args, **kwargs)
            elif kwargs['event'] == Analysis_Event.onDetailComplete:
                self.onDetailComplete(*args, **kwargs)
            elif kwargs['event'] == Analysis_Event.onListComplete:
                self.onListComplete(*args, **kwargs)
            elif kwargs['event'] == Analysis_Event.onIndexComplete:
                self.onIndexComplete(*args, **kwargs)
            return self.on(event.onListen, *args, **kwargs)
        except Exception as e:
            pass
        finally:
            self._lock.release()

    def onError(self, *args, **kwargs):
        try:
            self._lock.acquire()
            if len(args)>0:
                if args[0].getStatus== SPIDER_STATUS.HTTP_GATEWAY_TIME_OUT or args[0].getStatus== SPIDER_STATUS.HTTP_REQUEST_TIME:
                    # 如果访问超时缓存地址.
                    m_badRequest= BadRequest()
                    m_badRequest.Url= args[0].getCurrentUrl
                    m_badRequest.Source= None
                    m_badRequest.Key= args[0].getData['analysis_key']
                    m_badRequest.UpdateTime= datetime.datetime.now()
                    m_badRequest.toSave()
            return self.on(event.onListen, *args, **kwargs)
        except Exception as e:
            pass
        finally:
            self._lock.release()

    def onFingerprintComplete(self, *args, **kwargs):
        m_status = False
        if len(args) > 0:
            m_analysis = args[0]
            page_fingerprint = m_analysis.getFingerprint
            result = fingerprint().findOne(
                query={'url': m_analysis.getCurrentUrl})
            if result:
                if page_fingerprint == result['fingerprint']:
                    # 删除爬虫库数据
                    if self._fingerprint:
                        repeat_url = result['url']
                        m_L2 = L2()
                        m_L2.remove(
                            {'$or': [{'url': repeat_url}, {'sources': repeat_url}]})
                        m_L2 = None
                        # self._exclude_url[m_analysis.getNextUrl] =repeat_url
                        m_status = True
                    else:
                        # 当设置为不检查重复时
                        m_status = False
                else:
                    m_fingerprint = fingerprint()
                    m_fingerprint.update(
                        {
                            'url': m_analysis.getCurrentUrl
                        },
                        {
                            'fingerprint': page_fingerprint,
                            'lastupdatetime': datetime.datetime.now()
                        }
                    )
                    m_status = False
            else:
                m_fingerprint = fingerprint()
                m_fingerprint.Fingerprint = page_fingerprint
                m_fingerprint.Url = m_analysis.getCurrentUrl
                m_fingerprint.LastUpdateTime = datetime.datetime.now()
                m_fingerprint.toSave()
                m_status = False
        try:
            if not self._debug:
                self._lock.acquire()        
                self.on(event.onListen, *args, **kwargs)
        except Exception as e:
            pass
        finally:
            if not self._debug:
                self._lock.release()
        return m_status
